// BoostC version : 6.95
// Single user Full License for 1 node
// Target         : PIC18F4420
// Adappted to use with the nRF24L01
// by Martin Hug 2012
//
// The receiver UNIT is connected to a serial port through a terminal 
// and waits for incomming data.
////////////////////////////////////////////////////////////////////////////

#include <system.h>
#include "NG_RX_V1.h"
#include "serial.h"
#include "nrf24L01.h"
#include <PIC18f4420.h>

#ifdef _PIC18
#pragma DATA _CONFIG1H, _OSC_INTIO67_1H	  // Int OSC, Port RA6,RA7 I/O.
#pragma DATA _CONFIG2H, _WDT_OFF_2H       
#pragma DATA _CONFIG3H, _MCLRE_OFF_3H	
#endif _PIC18

#pragma CLOCK_FREQ 8000000			          // CLK 8Mhz.

#define PW_LED latd.5                     // PW LED.
#define PW_nRF latd.4                     // power up nRF24 (NOT NEEDED) 


void main()
{
	init();                                 // device and peripheral initialisation.                     
  unsigned char get_byte;
  char count = 0x02;
  
  nrf_rx_mode();                          // nRF receiver mode & power up.
  
	while (1){
     
     PW_LED_OnOff();                      // toggle LED  Off.

     while (nrf_irq_jack());              // waits for wireless data return.
     nrf_read_fifo();                     // reads data from the payload.
     while (count < 5){
       get_byte = nrf_get_byte(count);    // gets the data from data[] array.
       serial_printf(get_byte);           // print the return value to the terminal.
       count++;
       }
       count = 0x02;                      // set the pointer in spi_exchange function.
       nrf_reset_status();                // clear nrf24L01 status reg.
     }	
}
//
//

void init(){

  osccon = 01110000b;                     // internal oscillator at 8 MHz.
  while (!iofs);                          // iofs OSCCON stable bit.

  trisd = 00001000b;                      // PORTD,3 input is connected to IRQ nRF24L01 (active low)
  portd = 0x00;                           // change this if needed. (loock also at nrf25L01.h).

	adcon1 |= 0x0F;                         // All digital IO's, no analog ports required.
  PW_nRF = 1;                             // power for nRF24 (NOT NEEDED).

  delay_ms(100);
	
	  // Serial interface
	  serial_init(34);                      // (34) 57600 baud; DEBUG, before: BRG = 25 (19200 baud at 8MHz).
	  serial_printf("initializing serial port");
	  serial_print_lf();
	
	  // Wireless interface
	  nrf_init();
	  serial_printf("nRF24L01 config done...");
	  serial_print_lf();
	 return;
}

void PW_LED_OnOff()
{
	PW_LED = ~PW_LED;                       //toggle PW_LED (connected to PORTD,5).
}