// BoostC version : 6.95
// Single user Full License for 1 node
// Target         : PIC18F4420
// Adappted to use with the nRF24L01
// by Martin Hug 2012
//
// The transmitter UNIT scans the analog/ digital i/o of the PIC18F4420
// and send the data using the spi to the nRF124L01.
////////////////////////////////////////////////////////////////////////////

#include <system.h>
#include "NG_TX_V1.h"
#include "serial.h"
#include "nrf24L01.h"

#include <PIC18f4420.h>

#ifdef _PIC18
#pragma DATA _CONFIG1H, _OSC_INTIO67_1H     // Int OSC, Port RA6,RA7 I/O.
#pragma DATA _CONFIG2H, _WDT_OFF_2H       
#pragma DATA _CONFIG3H, _MCLRE_OFF_3H	
#endif _PIC18

#pragma CLOCK_FREQ 8000000                  // CLK 8Mhz.

#define PW_LED latd.5                       // PW LED.
#define PW_nRF latd.4                       // power up nRF24 (NOT NEEDED) 


void main()
{
	init();                                   // device and peripheral initialisation.                     
  unsigned char get_byte;
  char count = 0;
  short adc;
 

  nrf_tx_mode();                            // nRF transmitt mode & power up.
  nrf_reset_status();
   
  while (1){

   PW_LED_OnOff();                          // toggle LED  Off.
   tmr0if = 0x00;
    while (!tmr0if);                        // prescaler 1/64 = 8ms.
    count = scan_analog(count);             // scan analog inputs and puts the results into a buffer.
      adc = get_result();                   // reads analog buffer.
      if (adc >= 0x02){                     // Trash Hold of analog input.                     
      nrf_put_byte();                       // send data wireless with nRF24L01.
        while (nrf_irq_jack());             // waits until data has been sent.
	    PW_LED_OnOff();                     // toggle LED  On.
        nrf_reset_status();
        tmr0ie = true;                      // enable TMR0 overfolow interrupt.
        }
   
  }
	
}
//
//

void init(){

  char aux = 0x00;
  osccon = 01110000b;                       // internal oscillator at 8 MHz.
  while (!iofs);                            // iofs OSCCON stable bit.

  trisd = 00001000b;                        // PORTD,3 input is connected to IRQ nRF24L01 (active low)
  portd = 0x00;                             // change this if needed. (loock also at nrf25L01.h).
  trisa = 00111111b;

	adcon1 |= 0x0F;                           // All digital IO's, no analog ports required.
  PW_nRF = 1;                               // power for nRF24 (NOT NEEDED).

  t0con = 11000010b;                        // enable trm0, 8bit, 1/8 prescaler (fast enough).
  tmr0ie = true;                            // TIMER0 enable overfolw bit.
  clear_bit (intcon,1);                     // external interrupt.
	
  adcon1 = 00001001b;                       // RA0 - RA5 analog, rest digital I/O.
  adcon2 = 10100001b;                       // Result is right justified, conversion Ckl Fosc/8, TAD 8, 
  
	  // Serial interface
	  serial_init(34);                        // (34) 57600 baud; DEBUG before BRG = 103 (19200 baud at 8MHz).
	  serial_printf("initializing serial port");
	  serial_print_lf();
	
	  // Wireless interface
	  nrf_init();
	  serial_printf("nRF24L01 config done...");
	  serial_print_lf();
	 return;
}

void PW_LED_OnOff()
{
	PW_LED = ~PW_LED;                         // toggle PW_LED (connected to PORTD,5).
}